import unittest
from unittest.mock import patch

from tmc import points
from tmc.utils import load, load_module, reload_module, get_stdout, check_source
from functools import reduce
import os
import os.path
import textwrap
from random import choice, randint

exercise = 'src.store_personal_data'
function = "store_personal_data"
datafile = 'people.csv'

def clear_file():
    with open(datafile, "w"):
        pass

def get_content():
    with open(datafile) as f:
        return [x.replace("\n","") for x in f.readlines() if len(x.strip()) > 0] 

def format(f):
    return "\n".join(f)

def file_exists(f: str):
    try:
        open(f)
        return True
    except:
        return False

@points('1.store_personal_data')
class StorePersonalDataTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        with patch('builtins.input', side_effect=[AssertionError("Att be om input från användaren förväntades inte")]):
           cls.module = load_module(exercise, 'en')

    def test_0_main_ok(self):
        ok, line = check_source(self.module)
        message = """Koden för att testa funktionerna ska placeras inuti
if __name__ == "__main__":
block. Följande rad ska flyttas:
"""
        self.assertTrue(ok, message+line)

    def test_1_function_exists(self):
        try:
            from src.store_personal_data import store_personal_data
        except:
            self.fail("Din kod bör innehålla en funktion som heter store_personal_data(person: tuple)")
        try:
            store_personal_data(('Timothy Test', 45, 160.0))
        except Exception as e:
            self.fail("Funktionanropet store_personal_data(('Timothy Test', 45, 160.0)) fungerar inte:\n" + e)


    def test_2_write_one_person_1(self):
        test_case = ('Timothy Test', 45, 160.0)
        clear_file()
        correct = ["Timothy Test;45;160.0"]
        store_personal_data = load(exercise, function, 'en')
        try:
            store_personal_data(test_case)
        except OSError as ioe:
            self.fail(f"Funktionen ger ett fel med parametrar {test_case}: {ioe}")
        self.assertTrue(file_exists(datafile), f"Fil {datafile} kan inte hittas efter att funktionen har körts!")

        data = get_content()

        self.assertTrue(len(data) == 1, 
            f"Fil {datafile} ska innehålla 1 rad efter att funktionen anropades med tom fil med parametrarna {test_case} - nu finns det {len(data)} rader.")

        self.assertEqual(data, correct, f"Innehållet i filen bör vara \n{format(correct)}\nmen de är\n{format(data)}\nnär de anropas med parametrarna {test_case}")
                      
if __name__ == '__main__':
    unittest.main()
